/*

  This program was hacked together by Benjamin Haisch aka john_doe in 2005
  using parts of Residual's Smush playback code.  
  See readme.txt for more information.
  This file is in the public domain.
  Be aware that the files from Residual are under the LGPL license and not public domain!
  
  This is version 0.1

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <SDL/SDL.h>
#include <windows.h>

#include "smush.h"
#include "timer.h"

#include "aviwriter.h"

SDL_Surface *screen = NULL;

using namespace std;

enDebugLevels debugLevel = DEBUG_ALL;

int main(int argc, char *argv[]) {

  bool quit = false;
  bool writeAvi = false;
  bool fullscreen = false;
  byte *rgbBuffer;
  char *smushname;
  char aviname[256] = {0};

  if(argc < 2) {
    MessageBox(0,
      "Usage: grisp snm-filename [options]\n"
      "Options can be:\n"
      "f for full screen\n"
      "a to write an Avi file. the avi name will be the snm filename + \".avi\"\n",
      "GrSmush player Version 0.1", MB_OK);
    return 0;
  }

  smushname = argv[1];

  if(argc > 2) {
    for(int i = 0; i < argc; i++) {
      printf("%s\n", argv[i]);
      if(!stricmp(argv[i], "F")) fullscreen = true;
      else if(!stricmp(argv[i], "A")) {
        writeAvi = true;
        sprintf(aviname, "%s.avi", smushname);
      }
    }
  }

  printf("fullscreen = %d\n", fullscreen);
  printf("writeAvi = %d\n", writeAvi);
  printf("smushname = %s\n", smushname);
  printf("aviname = %s\n", aviname);

  SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_TIMER);

  // create objects
  Timer *timer = new Timer();
  g_mixer = new SoundMixer();
  Smush* smush = new Smush();

  g_mixer->setVolume(255);
  smush->play(smushname, 0, 0);
  smush->pause(true); // we need to wait for the user to select a codec
  
  if(writeAvi) {
    CreateAviFile(aviname, smush->getWidth(), smush->getHeight());
    SelectCompressor("smush.opt");
  }

  screen = SDL_SetVideoMode(640, 480, 16, SDL_SWSURFACE | (fullscreen ? SDL_FULLSCREEN : 0));

  if(fullscreen)
    SDL_ShowCursor(0);

  rgbBuffer = new byte[smush->getWidth() * smush->getHeight() * 4];

  smush->pause(false);
  
  while(smush->isPlaying() && !quit) {

    SDL_Event event;
    
    while(SDL_PollEvent(&event)) {
      switch(event.type) {
        case SDL_QUIT:
          quit = true;
        case SDL_KEYDOWN:
          quit = true;
      }
    }

    char buf[256];
    sprintf(buf, "Smush player: Frame %d/%d", smush->getFrame(), smush->getNbFrames());
    SDL_WM_SetCaption(buf, "Smush player");

    if(smush->isUpdateNeeded()) {

      SDL_LockSurface(screen);
      memcpy(screen->pixels, smush->getDstPtr(), smush->getWidth() * smush->getHeight() * 2);
      SDL_UnlockSurface(screen);

      if(writeAvi) {
        for(int y = 0; y < smush->getHeight(); y++)
          for(int x = 0; x < smush->getWidth(); x++) {
            uint16 pixel = *(uint16*)&(smush->getDstPtr()[(x+y*smush->getWidth())*2]);
            uint16 h = smush->getHeight();
            rgbBuffer[(x+(h-y)*smush->getWidth())*3+2] = ((pixel & 0xF800) >> 8); //r
            rgbBuffer[(x+(h-y)*smush->getWidth())*3+1] = ((pixel & 0x07E0) >> 3); //g
            rgbBuffer[(x+(h-y)*smush->getWidth())*3+0] = ((pixel & 0x001F) << 3); //b
          }
        AddFrame(rgbBuffer);
      }
    
    }
    
    SDL_Flip(screen);
    
    SDL_Delay(0);
    
    if(smush->getFrame() == smush->getNbFrames())
      break;
    
  }

  if(writeAvi)
    CloseAviFile();

  delete rgbBuffer;
  delete smush;
  delete g_mixer;
  delete timer;

  SDL_Quit();
  
  return 0;
}
